window.Gleam.initSearch([{"doc":"dnalg","title":"dnalg","content":"# dnalg\n\nDNAlg is a DNA sequence manipulation library. This is my implementation in Gleam.\n\n&gt; For a more portable version of this project, I will rewrite it in Go whenever I choose to learn Go.\n\n## Plans\n\n-   Restriction enzyme operations such as\n    -   Simulated digestion\n    -   Counting cut sites\n    -   Silent mutation of cut sites\n\n## Usage\n\nTo use `dnalg`, run it in the command line.\nThe easiest way to use it is by piping in a DNA sequence and pipe the output to a file.\n\nYou can also use the -i flag to specify an input file and the -o flag to specify an output file.\n\n```sh\ncat input.txt &gt; dnalg [subcommand] &gt; output.txt\n```\n\nThe input can be a text file, or in FASTA format. I will look at adding support for `.dna` and `.gb` files in the future.\n\n### Subcommands\n\n#### `silent-mutate`\n\n`silent-mutate` will silently mutate any restriction sites within the provided DNA sequence.\n\n-   Sends the new DNA sequence to stdout.\n\n```sh\ndnalg silent-mutate [-r|--restriction string]\n```\n\n## Roadmap\n\nYou can find the roadmap for this project here: [ROADMAP.md](ROADMAP.md)\n\n## Contributing\n\n### Running the project\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n\n### Adding functionality\n\n-   Use the built-in `cli/` module to add subcommands to the CLI command (when it is complete);\n-   Add new modules according to the flow outlines in the next section.\n\n### Module management\n\nModule structure is as follows:\n\n-   `core/` is for primitives and shared functionality;\n-   `actions/` are abstractions for core for use in commands;\n-   `commands/` are primary functions which run via the CLI.\n\nThis is done to keep the code clean and avoid circular dependencies.\nTry to have modules in `actions/` only import from `core/` and `commands/`\nonly import from `core/` or `actions/` where possible.\n\n---\n\nAll rights reserved. © 2024, Trevor Fox\n","url":"index.html"},{"doc":"dnalg","title":"dnalg","content":"# DNAlg Primitives\n\nWithin `core/` are the primitives that DNAlg uses for all actions.\n\n## Current Primitives\n\n### Codon\n\nA codon is a type which stores a string representation of a codon.\nThese can be created and validated using the `core/codon` module.\n\n### Amino Acid\n\nThis is simply a type which corresponds to each possible canonical, proteinogenic, amino acid.\n\nThese can be created using the `core/residue` module. However, this type is mostly internal, unless you are matching\na pattern upon them.\n\n### Residue\n\nA residue is a type which stores an Amino Acid, its codon, its letter representation, and its alternate codons.\n\nThese can be created using the `core/residue` module.\n\n### Sequence\n\nA sequence is a type which stores a string representation of a sequence.\nThese can be created and validated using the `core/sequence` module.\n\nNote that individual bases (A, C, G, and T) are not their own primitive. This is because they are already represented\nin DNAlg as strings (`&quot;A&quot;`, `&quot;C&quot;`, `&quot;G&quot;`, and `&quot;T&quot;`).\n\n## What is `core/tools`?\n\nThis is a module which shares functionality between every module. It must not import any other module,\nthus it can only use the Gleam standard library. To use this module, you must import it in your module and\nperform conversions on your data to the types that `core/tools` expects.\n","url":"ref/primitives.html"},{"doc":"dnalg","title":"dnalg","content":"# DNAlg Roadmap\n\nThis file can be used to track my progress.\nWhen I complete something, I will timestamp it so you can see if I am still working hard on this project.\n\n-   [ ] Create the api for restriction site analysis\n    -   [ ] Be able to take in custom restriction sites with a specific parseable format\n    -   [ ] Be able to take in a DNA sequence and return the number of restriction sites\n    -   [ ] Have a standard library of common restriction enzymes for synthetic biology standards (MoClo, BioBrick, etc.)\n-   [x] Create the sequence silent mutator to get rid of enzyme cut sites.\n    -   [ ] Make it get rid of every site, not just the first\n","url":"roadmap.html"},{"doc":"core/tools","title":"as_error","content":"pub fn as_error(msg: String) -&gt; String\n\nSynonyms:\ncore/tools.as_error\ncore/tools as_error","url":"core/tools.html#as_error"},{"doc":"core/tools","title":"normalize_sequence","content":"pub fn normalize_sequence(sequence: String) -&gt; String\n\nSynonyms:\ncore/tools.normalize_sequence\ncore/tools normalize_sequence","url":"core/tools.html#normalize_sequence"},{"doc":"core/tools","title":"core/tools","content":"","url":"core/tools.html"},{"doc":"core/codon","title":"Codon","content":"pub type Codon {\n  Codon(str: String)\n}\n\nCodon(str: String)\n\n\nSynonyms:\ncore/codon.Codon\ncore/codon Codon","url":"core/codon.html#Codon"},{"doc":"core/codon","title":"alternates","content":"pub fn alternates(\n  codon codon: Codon,\n  exclude exclude: List(String),\n) -&gt; List(String)\n\nSynonyms:\ncore/codon.alternates\ncore/codon alternates","url":"core/codon.html#alternates"},{"doc":"core/codon","title":"core/codon","content":"","url":"core/codon.html"},{"doc":"core/residue","title":"AminoAcid","content":"pub type AminoAcid {\n  Stop\n  Ala\n  Arg\n  Asn\n  Asp\n  Cys\n  Gln\n  Glu\n  Gly\n  His\n  Ile\n  Leu\n  Lys\n  Met\n  Phe\n  Pro\n  Ser\n  Thr\n  Trp\n  Tyr\n  Val\n}\n\nStop\n\n\nAla\n\n\nArg\n\n\nAsn\n\n\nAsp\n\n\nCys\n\n\nGln\n\n\nGlu\n\n\nGly\n\n\nHis\n\n\nIle\n\n\nLeu\n\n\nLys\n\n\nMet\n\n\nPhe\n\n\nPro\n\n\nSer\n\n\nThr\n\n\nTrp\n\n\nTyr\n\n\nVal\n\n\nSynonyms:\ncore/residue.AminoAcid\ncore/residue AminoAcid","url":"core/residue.html#AminoAcid"},{"doc":"core/residue","title":"Residue","content":"pub type Residue {\n  Residue(\n    residue: AminoAcid,\n    codon: String,\n    letter: String,\n    alternates: List(String),\n  )\n}\n\nResidue(\n  residue: AminoAcid,\n  codon: String,\n  letter: String,\n  alternates: List(String),\n)\n\n\nSynonyms:\ncore/residue.Residue\ncore/residue Residue","url":"core/residue.html#Residue"},{"doc":"core/residue","title":"from_codon","content":"pub fn from_codon(codon: Codon) -&gt; Residue\n\nSynonyms:\ncore/residue.from_codon\ncore/residue from_codon","url":"core/residue.html#from_codon"},{"doc":"core/residue","title":"core/residue","content":"","url":"core/residue.html"},{"doc":"core/sequence","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\ncore/sequence.main\ncore/sequence main","url":"core/sequence.html#main"},{"doc":"core/sequence","title":"validate_base","content":"pub fn validate_base(base: String) -&gt; Result(String, String)\n\nSynonyms:\ncore/sequence.validate_base\ncore/sequence validate_base","url":"core/sequence.html#validate_base"},{"doc":"core/sequence","title":"validate_sequence","content":"pub fn validate_sequence(sequence: String) -&gt; String\n Returns a string which accumulates invalid bases present in the sequence.\n If the sequence is valid, it will return an empty string.\n You can use this to validate a DNA sequence by matching on the empty\n string.\nSynonyms:\ncore/sequence.validate_sequence\ncore/sequence validate_sequence","url":"core/sequence.html#validate_sequence"},{"doc":"core/sequence","title":"core/sequence","content":"","url":"core/sequence.html"},{"doc":"actions/dna","title":"DnaParseError","content":"pub type DnaParseError {\n  UnknownParseError\n  InvalidBaseError(base: String)\n  InvalidLengthError(length: Int)\n  NoStartCodon\n}\n\nUnknownParseError\n\n\nInvalidBaseError(base: String)\n\n\nInvalidLengthError(length: Int)\n\n\nNoStartCodon\n\n\nSynonyms:\nactions/dna.DnaParseError\nactions/dna DnaParseError","url":"actions/dna.html#DnaParseError"},{"doc":"actions/dna","title":"DnaTranslation","content":"pub type DnaTranslation {\n  Translation(translation: List(String), trimmed: Int)\n  TranslationError(DnaParseError)\n}\n\nTranslation(translation: List(String), trimmed: Int)\n\n\nTranslationError(DnaParseError)\n\n\nSynonyms:\nactions/dna.DnaTranslation\nactions/dna DnaTranslation","url":"actions/dna.html#DnaTranslation"},{"doc":"actions/dna","title":"reverse_translate","content":"pub fn reverse_translate(residues: List(Residue)) -&gt; String\n\nSynonyms:\nactions/dna.reverse_translate\nactions/dna reverse_translate","url":"actions/dna.html#reverse_translate"},{"doc":"actions/dna","title":"translate","content":"pub fn translate(sequence: String) -&gt; DnaTranslation\n\nSynonyms:\nactions/dna.translate\nactions/dna translate","url":"actions/dna.html#translate"},{"doc":"actions/dna","title":"actions/dna","content":"","url":"actions/dna.html"},{"doc":"actions/translation","title":"isolate_residue","content":"pub fn isolate_residue(\n  seq_translation: List(Residue),\n  index: Int,\n) -&gt; #(List(Residue), List(Residue), List(Residue))\n\nSynonyms:\nactions/translation.isolate_residue\nactions/translation isolate_residue","url":"actions/translation.html#isolate_residue"},{"doc":"actions/translation","title":"actions/translation","content":"","url":"actions/translation.html"},{"doc":"commands/restriction","title":"MutationResult","content":"pub type MutationResult {\n  Mutated(new_sequence: String, new_residues: List(Residue))\n  CannotCompleteMutation\n}\n\nMutated(new_sequence: String, new_residues: List(Residue))\n\n\nCannotCompleteMutation\n\n\nSynonyms:\ncommands/restriction.MutationResult\ncommands/restriction MutationResult","url":"commands/restriction.html#MutationResult"},{"doc":"commands/restriction","title":"ResidueRange","content":"pub type ResidueRange {\n  ResidueRange(start: Int, end: Int)\n}\n\nResidueRange(start: Int, end: Int)\n\n\nSynonyms:\ncommands/restriction.ResidueRange\ncommands/restriction ResidueRange","url":"commands/restriction.html#ResidueRange"},{"doc":"commands/restriction","title":"RestrictionResult","content":"pub type RestrictionResult {\n  Digested(cuts: Int, fragments: List(String))\n  InvalidSequence\n}\n\nDigested(cuts: Int, fragments: List(String))\n\n\nInvalidSequence\n\n\nSynonyms:\ncommands/restriction.RestrictionResult\ncommands/restriction RestrictionResult","url":"commands/restriction.html#RestrictionResult"},{"doc":"commands/restriction","title":"silently_mutate","content":"pub fn silently_mutate(\n  sequence sequence: String,\n  recognition site: String,\n) -&gt; MutationResult\n\nSynonyms:\ncommands/restriction.silently_mutate\ncommands/restriction silently_mutate","url":"commands/restriction.html#silently_mutate"},{"doc":"commands/restriction","title":"try_digest","content":"pub fn try_digest() -&gt; Nil\n This function will asses the DNA sequence and return:\n - RestrictionResult::Digested(cuts, fragments)\n - RestrictionResult::NotDigested\n - RestrictionResult::InvalidSequence\nSynonyms:\ncommands/restriction.try_digest\ncommands/restriction try_digest","url":"commands/restriction.html#try_digest"},{"doc":"commands/restriction","title":"commands/restriction","content":"","url":"commands/restriction.html"},{"doc":"ffi/stdin","title":"read","content":"pub fn read() -&gt; String\n\nSynonyms:\nffi/stdin.read\nffi/stdin read","url":"ffi/stdin.html#read"},{"doc":"ffi/stdin","title":"ffi/stdin","content":"","url":"ffi/stdin.html"},{"doc":"cli/flags","title":"caps_flag","content":"pub fn caps_flag() -&gt; Flag(Bool)\n\nSynonyms:\ncli/flags.caps_flag\ncli/flags caps_flag","url":"cli/flags.html#caps_flag"},{"doc":"cli/flags","title":"restriction","content":"pub fn restriction() -&gt; Flag(String)\n\nSynonyms:\ncli/flags.restriction\ncli/flags restriction","url":"cli/flags.html#restriction"},{"doc":"cli/flags","title":"silent_splash","content":"pub fn silent_splash() -&gt; Flag(Bool)\n\nSynonyms:\ncli/flags.silent_splash\ncli/flags silent_splash","url":"cli/flags.html#silent_splash"},{"doc":"cli/flags","title":"cli/flags","content":"","url":"cli/flags.html"},{"doc":"cli","title":"cmd_silent_mutate","content":"pub fn cmd_silent_mutate() -&gt; Command(Nil)\n\nSynonyms:\ncli.cmd_silent_mutate\ncli cmd_silent_mutate","url":"cli.html#cmd_silent_mutate"},{"doc":"cli","title":"splash","content":"pub fn splash(silent: Bool) -&gt; Nil\n\nSynonyms:\ncli.splash\ncli splash","url":"cli.html#splash"},{"doc":"cli","title":"cli","content":"","url":"cli.html"},{"doc":"core/index","title":"SequenceIndex","content":"pub type SequenceIndex {\n  SequenceIndex(Int)\n}\n\nSequenceIndex(Int)\n\n\nSynonyms:\ncore/index.SequenceIndex\ncore/index SequenceIndex","url":"core/index.html#SequenceIndex"},{"doc":"core/index","title":"SequenceRange","content":"pub type SequenceRange {\n  SequenceRange(start: Int, end: Int)\n}\n\nSequenceRange(start: Int, end: Int)\n\n\nSynonyms:\ncore/index.SequenceRange\ncore/index SequenceRange","url":"core/index.html#SequenceRange"},{"doc":"core/index","title":"TranslationIndex","content":"pub type TranslationIndex {\n  TranslationIndex(Int)\n}\n\nTranslationIndex(Int)\n\n\nSynonyms:\ncore/index.TranslationIndex\ncore/index TranslationIndex","url":"core/index.html#TranslationIndex"},{"doc":"core/index","title":"TranslationRange","content":"pub type TranslationRange {\n  TranslationRange(start: Int, end: Int)\n}\n\nTranslationRange(start: Int, end: Int)\n\n\nSynonyms:\ncore/index.TranslationRange\ncore/index TranslationRange","url":"core/index.html#TranslationRange"},{"doc":"core/index","title":"into_seq","content":"pub fn into_seq(index: TranslationIndex) -&gt; SequenceIndex\n This is lossy. It will not return the exact range, but the range of the\n first base of the codon.\nSynonyms:\ncore/index.into_seq\ncore/index into_seq","url":"core/index.html#into_seq"},{"doc":"core/index","title":"into_seq_range","content":"pub fn into_seq_range(range: TranslationRange) -&gt; SequenceRange\n This is lossy. It will not return the exact range, but the range of the\n first base of each codon.\nSynonyms:\ncore/index.into_seq_range\ncore/index into_seq_range","url":"core/index.html#into_seq_range"},{"doc":"core/index","title":"into_trans","content":"pub fn into_trans(index: SequenceIndex) -&gt; TranslationIndex\n Translate a sequence index into a translation index.\nSynonyms:\ncore/index.into_trans\ncore/index into_trans","url":"core/index.html#into_trans"},{"doc":"core/index","title":"into_trans_range","content":"pub fn into_trans_range(range: SequenceRange) -&gt; TranslationRange\n Translate a sequence range into a translation range.\nSynonyms:\ncore/index.into_trans_range\ncore/index into_trans_range","url":"core/index.html#into_trans_range"},{"doc":"core/index","title":"core/index","content":"","url":"core/index.html"},{"doc":"dnalg","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\ndnalg.main\ndnalg main","url":"dnalg.html#main"},{"doc":"dnalg","title":"dnalg","content":"","url":"dnalg.html"}]);